#include <iostream>
#include <fstream>
#include <vector>
#include <queue>
#include <algorithm>
#include <set>
#include <cassert>

using namespace std;

#define MAX_VALUE 1e6
#define ull unsigned long long
#define cin fin
#define cout fout

ifstream fin("tinut.in");
ofstream fout("tinut.out");


vector<int> getPrimes(int N) {
    vector<int> types(N + 1, 0);
    vector<int> primes;

    //populate primes with prime numbers <= N using Eratosthenes sieve
    for(int i = 2; i <= N; i++) {
        if(types[i] == 0) {
            primes.push_back(i);
            for(int j = i; j <= N; j += i) {
                types[j] = 1;
            }
        }
    }

    return primes;
}

int getNumberOfDivisors_brute(ull x) {
    if (x == 1) {
        return 1;
    }
    if (x == 2) {
        return 2;
    }
    int noOfDivisors = 2;
    for (int d = 2; d * d <= x; ++d) {
        if (x % d == 0) {
            noOfDivisors++;
            if (d * d != x) {
                ++noOfDivisors;
            }
        }
    }

    return noOfDivisors;
}

int getNumberOfDivisors(ull X, const vector<int>& primes) {
    int noDivisors = 1;

    for(size_t i = 0; i < primes.size() && (unsigned long long)primes[i]*primes[i] <= X; i++) {
        int exponent = 1;
        while(X % primes[i] == 0) {
            X /= primes[i];
            exponent++;
        }
        noDivisors *= exponent;
    }
    if(X > 1)
        noDivisors *= 2;

    return noDivisors;
}

int getLengthOfShortestPath(const vector<vector<bool>>& accessible) {
    int N = accessible.size();
    vector<vector<int>> dist(N, vector<int>(N, -1));
    queue<pair<int, int>> q;

    q.push(make_pair(0, 0));
    dist[0][0] = 1;

    int diri[] = {-1, 0, 1, 0};
    int dirj[] = {0, 1, 0, -1};

    while(!q.empty()) {
        pair<int, int> current = q.front();
        q.pop();

        int x = current.first;
        int y = current.second;

        for(int i = 0; i < 4; i++) {
            int newx = x + diri[i];
            int newy = y + dirj[i];

            if(newx >= 0 && newx < N && newy >= 0 && newy < N && accessible[newx][newy] == 1) {
                if(dist[newx][newy] == -1) {
                    dist[newx][newy] = dist[x][y] + 1;
                    q.push(make_pair(newx, newy));
                }
            }
        }
    }

    return dist[N - 1][N - 1];
}

void solve1() {
    const vector<int>& primes = getPrimes(MAX_VALUE);

    int T;
    cin >> T;
    while(T--) {
        ull X;
        int N;
        cin >> X >> N;

        vector<vector<ull>> matrix(N, vector<ull>(N));

        for(int i = 0; i < N; i++) {
            for(int j = 0; j < N; j++) {
                cin >> matrix[i][j];
            }
        }

        assert(matrix[0][0] == 1);

        int noDivisorsOfX = getNumberOfDivisors(X, primes);

        vector<vector<int>> noDivisorsOfMatrix(N, vector<int>(N));
        for(int i = 0; i < N; i++) {
            for(int j = 0; j < N; j++) {
                noDivisorsOfMatrix[i][j] = getNumberOfDivisors(matrix[i][j], primes);
            }
        }

        vector<vector<bool>> accessibleForX(N, vector<bool>(N, 0));
        for(int i = 0; i < N; i++) {
            for(int j = 0; j < N; j++) {
                if(noDivisorsOfMatrix[i][j] <= noDivisorsOfX) {
                    accessibleForX[i][j] = 1;
                }
            }
        }


        int length = getLengthOfShortestPath(accessibleForX);
        cout << length << "\n";
    }
}

int getMinCostPath(const vector<vector<int>>& costMatrix) {
    int N = costMatrix.size();
    vector<vector<int>> dist(N, vector<int>(N, -1));

    struct nod {
        int x, y, cost;
        nod(int x, int y, int cost) {
            this->x = x;
            this->y = y;
            this->cost = cost;
        }

        bool operator<(const nod& other) const {
            if(cost == other.cost) {
                if(x == other.x)
                    return y < other.y;
                return x < other.x;
            }
            return cost < other.cost;
        }
    };

    set<nod> q;

    assert(costMatrix[0][0] != -1);

    q.insert(nod(0, 0, costMatrix[0][0]));

    dist[0][0] = costMatrix[0][0];

    int diri[] = {-1, 0, 1, 0};
    int dirj[] = {0, 1, 0, -1};

    while(!q.empty()) {


        nod current = *(q.begin());
        q.erase(q.begin());

        int x = current.x;
        int y = current.y;

        for(int i = 0; i < 4; i++) {
            int newx = x + diri[i];
            int newy = y + dirj[i];

            if(newx >= 0 && newx < N && newy >= 0 && newy < N && costMatrix[newx][newy] != -1) {
                if(dist[newx][newy] > dist[x][y] + costMatrix[newx][newy] || dist[newx][newy] == -1) {
                    auto it = q.find(nod(newx, newy, dist[newx][newy]));
                    if(it != q.end()) {
                        q.erase(it);
                    }
                    dist[newx][newy] = dist[x][y] + costMatrix[newx][newy];
                    q.insert(nod(newx, newy, dist[newx][newy]));
                }
            }
        }
    }

    return dist[N - 1][N - 1];
}

void solve2() {
    vector<int> primes = getPrimes(MAX_VALUE);

    int T;
    cin >> T;
    while(T--) {
        ull X;
        int N;
        cin >> X >> N;

        vector<vector<ull>> matrix(N, vector<ull>(N));

        for(int i = 0; i < N; i++) {
            for(int j = 0; j < N; j++) {
                cin >> matrix[i][j];
            }
        }

        int noDivisorsOfX = getNumberOfDivisors(X, primes);

        vector<vector<int>> noDivisorsOfMatrix(N, vector<int>(N));
        for(int i = 0; i < N; i++) {
            for(int j = 0; j < N; j++) {
                noDivisorsOfMatrix[i][j] = getNumberOfDivisors(matrix[i][j], primes);
            }
        }

        vector<vector<int>> costMatrix(N, vector<int>(N));
        for(int i = 0; i < N; i++) {
            for(int j = 0; j < N; j++) {
                costMatrix[i][j] = max(noDivisorsOfX - noDivisorsOfMatrix[i][j], -1);
            }
        }

        int cost = getMinCostPath(costMatrix);
        cout << cost << "\n";
    }
}

void solve3_brute() {
    int T;
    cin >> T;
    while(T--) {
        int N;
        cin >> N;

        vector<vector<ull>> matrix(N, vector<ull>(N));
        for (int i = 0; i < N; i++) {
            for (int j = 0; j < N; j++) {
                cin >> matrix[i][j];
            }
        }

        assert(matrix[0][0] == 0);

        ull left = 1;
        ull right = MAX_VALUE * MAX_VALUE;

        ull solutie = -1;

        for (ull candidate = left; candidate <= right; ++candidate) {
            ull X = candidate;
            vector<vector<bool>> accessibleForX(N, vector<bool>(N, 0));
            for (int i = 0; i < N; i++) {
                for (int j = 0; j < N; j++) {
                    if (matrix[i][j] < X) {
                        accessibleForX[i][j] = 1;
                    }
                }
            }

            int length = getLengthOfShortestPath(accessibleForX);

            if (length != -1) {
                solutie = X;
                return;
            }
        }

        cout << solutie << "\n";
    }
}

void solve3_binary() {
    int T;
    cin >> T;
    while(T--) {
        int N;
        cin >> N;

        vector<vector<ull>> matrix(N, vector<ull>(N));
        for(int i = 0; i < N; i++) {
            for(int j = 0; j < N; j++) {
                cin >> matrix[i][j];
            }
        }

        assert(matrix[0][0] == 0);

        ull left = 1;
        ull right = MAX_VALUE * MAX_VALUE;

        ull solutie = 0;

        for (int bit = 60; bit >= 0; --bit) {

            ull X = solutie + (1LL<<bit);
            vector<vector<bool>> accessibleForX(N, vector<bool>(N, 0));
            for(int i = 0; i < N; i++) {
                for(int j = 0; j < N; j++) {
                    if(matrix[i][j] < X) {
                        accessibleForX[i][j] = 1;
                    }
                }
            }

            int length = getLengthOfShortestPath(accessibleForX);

            if (length == -1) {
                solutie += (1LL<<bit);
            }
        }

        cout << solutie + 1 << "\n";
    }
}

void solve3() {
    int T;
    cin >> T;
    while(T--) {
        int N;
        cin >> N;

        vector<vector<ull>> matrix(N, vector<ull>(N));
        for(int i = 0; i < N; i++) {
            for(int j = 0; j < N; j++) {
                cin >> matrix[i][j];
            }
        }

        assert(matrix[0][0] == 0);

        ull left = 1;
        ull right = MAX_VALUE * MAX_VALUE;

        ull solutie = -1;

        while(left <= right) {
            ull mid = (left + right) / 2;

            ull X = mid;
            vector<vector<bool>> accessibleForX(N, vector<bool>(N, 0));
            for(int i = 0; i < N; i++) {
                for(int j = 0; j < N; j++) {
                    if(matrix[i][j] < X) {
                        accessibleForX[i][j] = 1;
                    }
                }
            }

            int length = getLengthOfShortestPath(accessibleForX);

            if(length != -1) {
                solutie = X;
                right = mid - 1;
            } else {
                left = mid + 1;
            }
        }

        cout << solutie << "\n";
    }
}

int main() {
    int C;
    cin >> C;

    switch(C) {
        case 1:
            solve1();
            break;
        case 2:
            solve2();
            break;
        case 3:
            solve3_binary();
            break;
        default:
            break;
    }

    return 0;
}
