#include <bits/stdc++.h>
#ifndef dbg 
#define dbg(...)
#endif

using namespace std;

vector<int> CountFactors(int n) {
  vector<int> count(n, 1);
  for (int i = 2; i < n; ++i) {
    for (int j = i; j < n; j += i) {
      ++count[j];
    }
  }
  
  return count;
}

int GetCountFactors(int64_t x, const vector<int> &primes, const vector<int> &count_factors) {
  if (x < (int)count_factors.size()) return count_factors[x];
  int cnt = 1;
  for (int p : primes) {
    int e = 0;
    while (x % p == 0) {
      x /= p;
      e += 1;
    }
    cnt *= e + 1;
    if (x < (int)count_factors.size()) return cnt * count_factors[x];
  }
  return 2 * cnt;
}

const int INF = (int)1e9 + 10;
const int dx[] = {-1, 0, +1, 0};
const int dy[] = {0, +1, 0, -1};

int BFS(const vector<vector<int64_t>> &a, 
  const function<bool(int64_t)> &is_accesible,
  const function<int64_t(int64_t)> &get_cost) {

  int n = a.size();

  auto inside = [&](int x, int y) {
    return 0 <= x && x < n && 0 <= y && y < n;
  };

  static vector<pair<int, int>> q = {{0, 0}}; 
  static vector<vector<int>> cost(n, vector<int>(n, INF));


  q.assign(1, {0, 0});
  cost.assign(n, vector<int>(n, INF));
  cost[0][0] = get_cost(a[0][0]);

  for (int it = 0; it < (int)q.size(); ++it) {
    int x, y; tie(x, y) = q[it];
    for (int d = 0; d < 4; ++d) {
      int nx = x + dx[d];
      int ny = y + dy[d];
      if (!inside(nx, ny) || !is_accesible(a[nx][ny])) continue;

      int ncost = cost[x][y] + get_cost(a[nx][ny]);
      if (cost[nx][ny] > ncost) {
        cost[nx][ny] = ncost;
        q.emplace_back(nx, ny); 
      }
    }
  }

  return cost[n - 1][n - 1];
}

int Solve1(int64_t fact_x, const vector<vector<int64_t>> &a, 
  const vector<int> &primes, const vector<int> &count_factors) {
  
  int factors = GetCountFactors(fact_x, primes, count_factors); 
  
  int ans = BFS(a, [&](int64_t x) {
    return factors >= x; 
  }, [&](int64_t) {
    return 1;
  });
  if (ans == INF) ans = -1;
  return ans;
}

int Solve2(int64_t fact_x, const vector<vector<int64_t>> &a,
  const vector<int> &primes, const vector<int> &count_factors) {

  int factors = GetCountFactors(fact_x, primes, count_factors); 
  
  int ans = BFS(a, [&](int64_t x) {
    return factors >= x; 
  }, [&](int64_t x) {
    return factors - x;
  });
  if (ans == INF) ans = -1;
  return ans;
}

int64_t Solve3(const vector<vector<int64_t>> &a) {
  auto Check = [&](int64_t val) {
    return INF != BFS(a, [&](int64_t x) {
      return val > x;
    }, [&](int64_t) {
      return 0;
    });
  };

  int64_t fact = -1, adv = 1; 
  for (int64_t step = (1LL << 40); step; step /= 2) {
    if (!Check(fact + step)) {
      fact += step; 
    } else adv = 0;
  }

  return fact + 1;
}

int64_t SolveTest(int c, const vector<int> &primes, const vector<int> &count_factors) {
  int64_t fact_x = -1;
  if (c <= 2) {
    cin >> fact_x;
    assert(1 <= fact_x && fact_x <= (int64_t)1e12);
  }

  int n; cin >> n;
  assert(1 <= n && n <= 500);
  vector<vector<int64_t>> a(n, vector<int64_t>(n));
  int fact_x_factors = GetCountFactors(fact_x, primes, count_factors);
  for (int i = 0; i < n; ++i) {
    for (int j = 0; j < n; ++j) {
      cin >> a[i][j];
      assert(0 <= a[i][j] && a[i][j] <= (int64_t)1e12);
      if (c <= 2) {
        assert(a[i][j] != 0);
        a[i][j] = GetCountFactors(a[i][j], primes, count_factors);
        assert(fact_x_factors >= a[0][0]);
      }
    }
  }

  if (c == 1) {
    return Solve1(fact_x, a, primes, count_factors);
  } 
  if (c == 2) {
    return Solve2(fact_x, a, primes, count_factors);
  }
  return Solve3(a);
}

int main() {
  ios::sync_with_stdio(0);
  cin.tie(0);
#ifndef LOCAL_DEFINE
  freopen("tinut.in", "r", stdin);
  freopen("tinut.out", "w", stdout);
#endif

  const int N = (int)1e6 + 10;
  auto count_factors = CountFactors(N);
  vector<int> primes;
  for (int i = 2; i < N; ++i) {
    if (count_factors[i] == 2) primes.emplace_back(i);
  }

  int c; cin >> c;
  assert(1 <= c && c <= 3);
  int t; cin >> t;
  assert(1 <= t && t <= 10);
  while (t--) {
    cout << SolveTest(c, primes, count_factors) << '\n';
  }
}
